/**************************************/
/* thining, squelettisation */
/* Tarik Fdil <tfdil@sagma.ma> */
/* april 2005 */
/* Licence GPL cf. http://www.gnu.org */
/**************************************/
#include <tiffio.h>
#include <stdlib.h>
#define max(a,b) ((a)>(b))?(a):(b)
#define min(a,b) ((a)<(b))?(a):(b)
#define isblack(a,b) ((a)<0||(b)<0||(a)>=w||(b)>=h)?0:(buf[(b)*w+(a)] & 0x00FFFFFF)?0:1
#define iswhite(a,b) ((a)<0||(b)<0||(a)>=w||(b)>=h)?1:(buf[(b)*w+(a)] & 0x00FFFFFF ^ 0x00FFFFFF)?0:1
typedef char matrix[3][3];

static int is_this_pixel_required(uint32 w,uint32 h,uint32 x, uint32 y,uint32 *buf);

void thinning(uint32 w, uint32 h,uint32 *buf)
{
int n,more,x,y,c,white=0xffffffff;
enum direction {north,east,south,west,ne,se,nw,sw,none} dir,dirpix;


do {
      	more=0;
	for(dir=north; dir<=sw; dir++) {
      		for (x=0; x<w; x++)
		for (y=0; y<h; y++)
		{	if(iswhite(x,y)) continue; 
			dirpix = (x==0)? west: (x==w-1)? east : (y==0)? north : (y==h-1)? south:
				 (iswhite(x,y-1))?north:(iswhite(x+1,y))?east:(iswhite(x,y+1))?south: 
				 (iswhite(x-1,y))?west:(iswhite(x+1,y+1))?ne:(iswhite(x+1,y+1))?se: 
				 (iswhite(x-1,y+1))?sw:(iswhite(x-1,y-1))?nw:none;
			if(dir != dirpix) continue;  
	 		if (!is_this_pixel_required(w,h,x,y,buf)) { 
				buf[y*w+x]=white;
				more=1;
			}
		}
	}
} while(more);


} /* thinning */

void propagation(matrix mat, char color, char type_connexite)
{
int x,y,i,j,neighbour;

for(x=0; x<3; x++) 
	for(y=0; y<3; y++) {
		if(mat[x][y] == 'v') {
			for(i=0; i<3; i++) 
				for(j=0; j<3; j++) {
					switch(type_connexite) {
						case 8 : 
							neighbour = abs(i-x)<2 && abs(j-y)<2 ; 
							break;
						case 4 :
							neighbour = abs(i-x)==1 && j==y || i==x && abs(j-y)==1 ;
							break;
					}
					if(neighbour && mat[i][j]==color) mat[i][j]='v';
				}
		}
	}
}


int get_connex_number(matrix m, int color)
{ int x,y,connex=0;
matrix mat;

for(y=0; y<3; y++) 
	for(x=0 ; x<3; x++) mat[x][y] = m[x][y];

for(y=0; y<3; y++) 
	for(x=0 ; x<3; x++) {
		if (mat[x][y]==((color=='b')? 'w' : 'b') || mat[x][y]=='v') continue;
		connex++;
		mat[x][y]='v';
		propagation(mat,color,(color=='w')? 4 : 8);
	}

return connex;
}

static int is_this_pixel_required(uint32 w,uint32 h,uint32 x0, uint32 y0,uint32 *buf)
{
matrix matw,matb;
int x,y,ww,bw,wb,bb;

for(y= -1; y<2; y++) {
	for(x= -1; x<2; x++) {
		matb[x+1][y+1] = (isblack(x0+x,y0+y))?'b':'w';
		matw[x+1][y+1] = matb[x+1][y+1];
	}
}
matw[1][1]='w';

ww = get_connex_number(matw,'w');
bw = get_connex_number(matb,'w'); 
wb = get_connex_number(matw,'b');
bb = get_connex_number(matb,'b');

/* if( wb == 0 && (ww != bw || wb != bb) ) return 1; else return 0; */

if(bw == 1 && bb == 1) return 0; else return 1;

} /* is_this_pixel_required */
